package com.totopia.server.modules.user.entity;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.totopia.server.commons.data.entity.DateAuditEntity;
import lombok.experimental.SuperBuilder;

import javax.persistence.*;
import java.util.Date;
import java.util.Set;

@Entity
@Table(name = "users")
@SuperBuilder
public class UserEntity extends DateAuditEntity {
  private static final long serialVersionUID = 8891163223262220481L;

  @Id
  @GeneratedValue(generator = "users_generator")
  @SequenceGenerator(name = "users_generator", sequenceName = "users_sequence", initialValue = 1)
  private Long id;

  @Column(name = "username", unique = true, nullable = false, length = 150)
  private String username;

  @Basic
  @Column(name = "password", nullable = false, length = 100)
  @JsonIgnore
  private String password;

  @Basic
  @Column(name = "nickname", nullable = false, length = 150)
  private String nickname;

  @Basic
  @Column(name = "email", nullable = false, length = 100)
  private String email;

  @Basic
  @Column(name = "block", nullable = false)
  private Boolean block = false;

  @Basic
  @Column(name = "send_email", nullable = false)
  private Boolean sendEmail = true;

  @Basic
  @Column(name = "activation", nullable = true, length = 100)
  private String activation;

  @Basic
  @Temporal(TemporalType.TIMESTAMP)
  @Column(name = "last_reset_time", nullable = true)
  private Date lastResetTime;

  @Basic
  @Column(name = "reset_count", nullable = false)
  private Long resetCount = 0L;

  @Basic
  @Column(name = "otp_key", nullable = true, length = 1000)
  private String otpKey;

  @Basic
  @Column(name = "otep", nullable = true, length = 1000)
  private String otep;

  @Basic
  @Column(name = "require_reset", nullable = true)
  private Boolean requireReset = false;

  @ManyToMany(fetch = FetchType.EAGER)
  @JoinTable(name = "user_roles", joinColumns = @JoinColumn(name = "user_id"), inverseJoinColumns = @JoinColumn(name = "role_id"))
  private Set<RoleEntity> roles;

  public UserEntity(String username, String password, String nickname, String email, Boolean block, Boolean sendEmail, String activation, Date lastResetTime, Long resetCount, String otpKey, String otep, Boolean requireReset, Set<RoleEntity> roles) {
    this.username = username;
    this.password = password;
    this.nickname = nickname;
    this.email = email;
    this.block = block;
    this.sendEmail = sendEmail;
    this.activation = activation;
    this.lastResetTime = lastResetTime;
    this.resetCount = resetCount;
    this.otpKey = otpKey;
    this.otep = otep;
    this.requireReset = requireReset;
    this.roles = roles;
  }

  public UserEntity() {
  }

  public Long getId() {
    return id;
  }

  public void setId(Long id) {
    this.id = id;
  }

  public String getUsername() {
    return this.username;
  }

  public String getPassword() {
    return this.password;
  }

  public String getNickname() {
    return this.nickname;
  }

  public String getEmail() {
    return this.email;
  }

  public Boolean getBlock() {
    return this.block;
  }

  public Boolean getSendEmail() {
    return this.sendEmail;
  }

  public String getActivation() {
    return this.activation;
  }

  public Date getLastResetTime() {
    return this.lastResetTime;
  }

  public Long getResetCount() {
    return this.resetCount;
  }

  public String getOtpKey() {
    return this.otpKey;
  }

  public String getOtep() {
    return this.otep;
  }

  public Boolean getRequireReset() {
    return this.requireReset;
  }

  public Set<RoleEntity> getRoles() {
    return this.roles;
  }

  public void setUsername(String username) {
    this.username = username;
  }

  public void setPassword(String password) {
    this.password = password;
  }

  public void setNickname(String nickname) {
    this.nickname = nickname;
  }

  public void setEmail(String email) {
    this.email = email;
  }

  public void setBlock(Boolean block) {
    this.block = block;
  }

  public void setSendEmail(Boolean sendEmail) {
    this.sendEmail = sendEmail;
  }

  public void setActivation(String activation) {
    this.activation = activation;
  }

  public void setLastResetTime(Date lastResetTime) {
    this.lastResetTime = lastResetTime;
  }

  public void setResetCount(Long resetCount) {
    this.resetCount = resetCount;
  }

  public void setOtpKey(String otpKey) {
    this.otpKey = otpKey;
  }

  public void setOtep(String otep) {
    this.otep = otep;
  }

  public void setRequireReset(Boolean requireReset) {
    this.requireReset = requireReset;
  }

  public void setRoles(Set<RoleEntity> roles) {
    this.roles = roles;
  }

  public String toString() {
    return "UserEntity(username=" + this.getUsername() + ", password=" + this.getPassword() + ", nickname=" + this.getNickname() + ", email=" + this.getEmail() + ", block=" + this.getBlock() + ", sendEmail=" + this.getSendEmail() + ", activation=" + this.getActivation() + ", lastResetTime=" + this.getLastResetTime() + ", resetCount=" + this.getResetCount() + ", otpKey=" + this.getOtpKey() + ", otep=" + this.getOtep() + ", requireReset=" + this.getRequireReset() + ", roles=" + this.getRoles() + ")";
  }

  public boolean equals(final Object o) {
    if (o == this) return true;
    if (!(o instanceof UserEntity)) return false;
    final UserEntity other = (UserEntity) o;
    if (!other.canEqual((Object) this)) return false;
    final Object this$username = this.getUsername();
    final Object other$username = other.getUsername();
    if (this$username == null ? other$username != null : !this$username.equals(other$username)) return false;
    final Object this$password = this.getPassword();
    final Object other$password = other.getPassword();
    if (this$password == null ? other$password != null : !this$password.equals(other$password)) return false;
    final Object this$nickname = this.getNickname();
    final Object other$nickname = other.getNickname();
    if (this$nickname == null ? other$nickname != null : !this$nickname.equals(other$nickname)) return false;
    final Object this$email = this.getEmail();
    final Object other$email = other.getEmail();
    if (this$email == null ? other$email != null : !this$email.equals(other$email)) return false;
    final Object this$block = this.getBlock();
    final Object other$block = other.getBlock();
    if (this$block == null ? other$block != null : !this$block.equals(other$block)) return false;
    final Object this$sendEmail = this.getSendEmail();
    final Object other$sendEmail = other.getSendEmail();
    if (this$sendEmail == null ? other$sendEmail != null : !this$sendEmail.equals(other$sendEmail)) return false;
    final Object this$activation = this.getActivation();
    final Object other$activation = other.getActivation();
    if (this$activation == null ? other$activation != null : !this$activation.equals(other$activation))
      return false;
    final Object this$lastResetTime = this.getLastResetTime();
    final Object other$lastResetTime = other.getLastResetTime();
    if (this$lastResetTime == null ? other$lastResetTime != null : !this$lastResetTime.equals(other$lastResetTime))
      return false;
    final Object this$resetCount = this.getResetCount();
    final Object other$resetCount = other.getResetCount();
    if (this$resetCount == null ? other$resetCount != null : !this$resetCount.equals(other$resetCount))
      return false;
    final Object this$otpKey = this.getOtpKey();
    final Object other$otpKey = other.getOtpKey();
    if (this$otpKey == null ? other$otpKey != null : !this$otpKey.equals(other$otpKey)) return false;
    final Object this$otep = this.getOtep();
    final Object other$otep = other.getOtep();
    if (this$otep == null ? other$otep != null : !this$otep.equals(other$otep)) return false;
    final Object this$requireReset = this.getRequireReset();
    final Object other$requireReset = other.getRequireReset();
    if (this$requireReset == null ? other$requireReset != null : !this$requireReset.equals(other$requireReset))
      return false;
    final Object this$roles = this.getRoles();
    final Object other$roles = other.getRoles();
    if (this$roles == null ? other$roles != null : !this$roles.equals(other$roles)) return false;
    return true;
  }

  protected boolean canEqual(final Object other) {
    return other instanceof UserEntity;
  }

  public int hashCode() {
    final int PRIME = 59;
    int result = 1;
    final Object $username = this.getUsername();
    result = result * PRIME + ($username == null ? 43 : $username.hashCode());
    final Object $password = this.getPassword();
    result = result * PRIME + ($password == null ? 43 : $password.hashCode());
    final Object $nickname = this.getNickname();
    result = result * PRIME + ($nickname == null ? 43 : $nickname.hashCode());
    final Object $email = this.getEmail();
    result = result * PRIME + ($email == null ? 43 : $email.hashCode());
    final Object $block = this.getBlock();
    result = result * PRIME + ($block == null ? 43 : $block.hashCode());
    final Object $sendEmail = this.getSendEmail();
    result = result * PRIME + ($sendEmail == null ? 43 : $sendEmail.hashCode());
    final Object $activation = this.getActivation();
    result = result * PRIME + ($activation == null ? 43 : $activation.hashCode());
    final Object $lastResetTime = this.getLastResetTime();
    result = result * PRIME + ($lastResetTime == null ? 43 : $lastResetTime.hashCode());
    final Object $resetCount = this.getResetCount();
    result = result * PRIME + ($resetCount == null ? 43 : $resetCount.hashCode());
    final Object $otpKey = this.getOtpKey();
    result = result * PRIME + ($otpKey == null ? 43 : $otpKey.hashCode());
    final Object $otep = this.getOtep();
    result = result * PRIME + ($otep == null ? 43 : $otep.hashCode());
    final Object $requireReset = this.getRequireReset();
    result = result * PRIME + ($requireReset == null ? 43 : $requireReset.hashCode());
    final Object $roles = this.getRoles();
    result = result * PRIME + ($roles == null ? 43 : $roles.hashCode());
    return result;
  }

//  @Builder
//  public UserEntity(String username, String password, String nickname, String email, Date createAt, Date updateAt) {
//    super(createAt, updateAt);
//    this.username = username;
//    this.password = password;
//    this.nickname = nickname;
//    this.email = email;
//  }
}

// 아이디
// 로그인 아이디
// 로그인 패스워드
// 이메일
// 닉네임
// 은행명
// 계좌번호
// 예금주
// 추천인
// 추천수
// 게시판 제한 여부
// 쿠폰
// 충전방식
// 비고
// 종목별 베팅제한
// 상태표기
// 휴대폰번호
// 추천권한 여부
// 가입상태
// 소속
// 레벨
// 보유머니
// 포인트
// 회원상태
// 룰렛개수
// 계좌순번
// 가입날짜
// 최근접속 날짜
// 가입 아이피
// 최근 접속 아이피
// 베팅 알림
// API 연결
